//set pins
const int RUNNING_PIN = 1;
const int HEARTBEAT_PIN = 10;
const int RELAY_PIN = 2; //Change to 2 for Prod, 11 for Debug
const int MANUAL_PIN = 3;
const int MANUAL_LED_PIN = 4;
const int DELAY_PIN = A1;
const int ACC_PIN = 11;

const long MAX_TIMEOUT = 60000; //60 seconds grace between heartbeats

//local variables
boolean triggerOn = false;
int manualPinValue = 0;
long lastDelayAmount = 0;
elapsedMillis sinceHeartbeat = 0;
String inputString = "";
boolean stringComplete = false;
boolean gotHeartBeat = false;

void setup() {

  pinMode(RUNNING_PIN, OUTPUT); //red
  pinMode(HEARTBEAT_PIN, OUTPUT); //blue
  pinMode(MANUAL_LED_PIN, OUTPUT); //amber
  pinMode(RELAY_PIN, OUTPUT);  //green

  pinMode(ACC_PIN, INPUT);
  pinMode(MANUAL_PIN, INPUT);
  
  //do funky LED dance  
  digitalWrite(RUNNING_PIN, HIGH);
  delay(250);
  digitalWrite(HEARTBEAT_PIN, HIGH);
  delay(250);
  digitalWrite(MANUAL_LED_PIN, HIGH);
  
  delay(1000);
  
  digitalWrite(RUNNING_PIN, LOW);
  delay(250);
  digitalWrite(HEARTBEAT_PIN, LOW);
  delay(250);
  digitalWrite(MANUAL_LED_PIN, LOW);
  
  //set buffer size
  inputString.reserve(128);
  
  //open serial port
  Serial.begin(9600);

}

void loop() {
  
  //turn off power LED, we're processing
  digitalWrite(RUNNING_PIN, LOW);
  
  int accOn = digitalRead(ACC_PIN);
    
  triggerOn = false; //assume timed out
  
  manualPinValue = digitalRead(MANUAL_PIN); //override?
  digitalWrite(MANUAL_LED_PIN, manualPinValue); //show it
  
  int delayValue = analogRead(DELAY_PIN);
  long msDelayAmount = map(delayValue, 0, 1023, 5000, MAX_TIMEOUT); //ms delay

  if (abs(lastDelayAmount - msDelayAmount) > 200) //if delta is greater than 200ms
  {
    lastDelayAmount = msDelayAmount;
    Serial.print("$DELAY$");
    Serial.print(msDelayAmount);
    Serial.print("\n");
  }
  
  long msTimeLeft =  msDelayAmount - sinceHeartbeat;
  
  int intensity = 0; //asume off
  if (msTimeLeft > 0 && gotHeartBeat) //still time left, calculate LED intensity
    intensity = map((msTimeLeft / (double)msDelayAmount) * 100.0, 0, 100, 0, 255);
  
  if (manualPinValue == HIGH) triggerOn = true; //forced on
  
  if (stringComplete) //command received
  { 
    
    if (inputString == "PING" && accOn == HIGH) //heartbeat
    {
        sinceHeartbeat = 0;
        triggerOn = true;
        gotHeartBeat = true;
    }
    else if (inputString == "DUMP") //debug info
    {
       Serial.print("Manual Pin: ");
       Serial.print(manualPinValue);
       Serial.print("\tDelay Read: ");
       Serial.print(delayValue);
       Serial.print("\tDelay Calc: ");
       Serial.print(msDelayAmount);
       Serial.print("\r\nMs since HB: ");
       Serial.print(sinceHeartbeat);
       Serial.print("\tms left: ");
       Serial.print(msTimeLeft);
       Serial.print("\tIntensity: ");
       Serial.println(intensity);
    }
    else if (inputString == "DELAY") //delay query
    {
      Serial.print("$");
      Serial.print("DELAY$");
      Serial.print(msDelayAmount);
      Serial.print("\n");
    }
    else if (inputString == "KILL") {
      triggerOn = false;
      gotHeartBeat = false;
      sinceHeartbeat = msDelayAmount; 
    }
    else
    {
      Serial.println("Unknown: \'");
      Serial.println(inputString);
      Serial.println("\'\n");
    }
    
    inputString = "";
    stringComplete = false;

  }

  
  if (triggerOn) //forced on
     digitalWrite(RELAY_PIN, HIGH);
  else if (sinceHeartbeat > msDelayAmount) //has expired?
     digitalWrite(RELAY_PIN, LOW);      
  
  //update heartbeat LED brightness
  analogWrite(HEARTBEAT_PIN, intensity);    
  
  delay(20);
  digitalWrite(RUNNING_PIN, HIGH); //flicker power led to show we're alive
  delay(20);
 
  while (Serial.available()) 
  {
   
   char inChar = (char)Serial.read();
   Serial.print(inChar); //echo
   
   if (inChar == '\n' || inChar == '\r') 
      stringComplete = true;
   else
      inputString += inChar;   
    
    if (inputString.length() == 128 && !stringComplete) { //truncate to prevent overflow
      inputString = inputString.substring(1);
    }
    
  }
  
}

